package com.caimei.www.service.page;

import com.caimei.www.pojo.JsonModel;
import com.caimei.www.pojo.document.OssAuthorization;
import org.springframework.http.ResponseEntity;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpResponse;
import reactor.core.publisher.Mono;

/**
 * Description
 *
 * @author : plf
 * @date : 2020/11/10
 */
public interface DocumentAuthService {

    /**
     * 获取cookie数据,判断是否登录
     *
     * @param authorizationMobile cookie中的数据
     * @return
     */
    Boolean getAuthorizationCookie(String authorizationMobile);

    /**
     * 获取图形验证码
     */
    Mono<JsonModel> getImgVerifyCode();

    /**
     * oss短信登录,发送短信验证
     *
     * @param mobile  手机号
     * @param imgCode 图形验证码
     * @param token   图形token
     * @return
     */
    Mono<JsonModel> ossNote(String mobile, String imgCode, String token);

    /**
     * oss手机短信验证码登录
     *
     * @param authorization 用户信息
     * @return
     */
    ResponseEntity<JsonModel> mobileCodeLogin(OssAuthorization authorization);

    /**
     * 资料列表
     *
     * @param name     商品名称，资料库名称，资料名称
     * @param pageNum  分页参数
     * @param pageSize 分页参数
     * @return
     */
    Mono<JsonModel> dataList(String name, Integer pageNum, Integer pageSize);

    /**
     * 资料详情
     *
     * @param pdfId 文件id
     * @return
     */
    Mono<JsonModel> dataDetails(Integer pdfId);

    /**
     * 更多资料
     *
     * @param archiveId 资料库id
     * @return
     */
    Mono<JsonModel> moreData(Integer archiveId);


    /**
     * 文件下载
     *
     * @param pdfId 文件id
     * @return
     */
    Mono<Void> downFile(Integer pdfId, ServerHttpRequest request, ServerHttpResponse response);

    /**
     * 定时删除服务器文件
     *
     * @return
     */
    void deleteFile();

    Mono<JsonModel> downFileAll(Integer pdfId);
}
